<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'mdl/contract.php';

class contract_dao extends dao
{
	public $key = 'contract_id';
	public $foreign_keys = 'client_id,service_id';
	protected $model = 'contract';
	protected $table = 'contracts';
	protected $fields_rel_all = array(
		'handle' => array('contracts' => 'contract_id'),
		'client_username' => array('clients' => 'username'),
		'client_name' => array('clients' => 'name'),
		'service_name' => array('services' => 'name'),
		'service_price' => array('services' => 'price'),
		'service_payment' => array('services' => 'payment'),
	);
	protected $fields_count = array(
		'all_invoices_count' => array('all_invoices' => 'invoice_id')
	);
	protected $fields_sum = array(
		'service_price' => array('services' => 'price')
	);
	protected $fields_max = array(
		'invoice_date' => array('invoices' => 'date'),
		'invoice_period_end' => array('invoices' => 'period_end')
	);
	protected $select_sql = "DATE_FORMAT(contracts.start, '%Y-%m') AS month,
		DATE_FORMAT(contracts.start, '%Y-%m-%d') AS start_date,
		DATE_FORMAT(contracts.start, '%H:%i:%s') AS start_time,
		DATE_FORMAT(contracts.end, '%Y-%m-%d') AS end_date,
		DATE_FORMAT(contracts.end, '%H:%i:%s') AS end_time";
	protected $left_joins = array(
		'clients.client_id' => 'contracts.client_id',
		'services.service_id' => 'contracts.service_id'
	);
	protected $joins_sql = '
		LEFT JOIN invoices ON (
			invoices.contract_id = contracts.contract_id
			AND invoices.service_id = contracts.service_id )
		LEFT JOIN invoices all_invoices ON (
			invoices.contract_id = contracts.contract_id )';
	protected $search_fields = 'clients.username,clients.name,services.name';
	public $order_by = 'clients.name,contract_id';
	public $group_by = 'contract_id';

	public function __construct()
	{
		parent::__construct();
	}

	public function fetch_cron($a = false, $b = false)
	{
		$config = config::get_instance();
		$models = array();
		$sql  = $this->sql_select().',
			MAX(invoices.date) AS last_invoice_date,
			MAX(payments.date) AS last_payment_date,
			SUM(payments.value) AS total_payments_value,
			SUM(unpaid_invoices.value) AS total_unpaid_value,
			MAX(unpaid_invoices.date) AS last_unpaid_invoice_date'
			.$this->sql_from().'
			INNER JOIN clients ON (
				clients.client_id = contracts.client_id )
			INNER JOIN services ON (
				services.service_id = contracts.service_id )
			LEFT JOIN invoices ON (
				invoices.contract_id = contracts.contract_id
				AND invoices.status != \'C\' )
			LEFT JOIN invoices unpaid_invoices ON (
				unpaid_invoices.contract_id = contracts.contract_id
				AND unpaid_invoices.date < \''.date('Ymd', mktime(0, 0, 0, date('m'), date('d') - $config->payment_delay, date('Y'))).'\'
				AND ( unpaid_invoices.value > (
						SELECT SUM(p.value)
						FROM payments p
						WHERE p.invoice_id = unpaid_invoices.invoice_id )
					OR  (
						SELECT SUM(p.value)
						FROM payments p
						WHERE p.invoice_id = unpaid_invoices.invoice_id ) IS NULL )
				AND unpaid_invoices.status != \'C\' )
			LEFT JOIN payments ON (
				payments.invoice_id = unpaid_invoices.invoice_id )'
			.$this->sql_group_by()
			.$this->sql_order_by();

		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function fetch_overdue()
	{
		$models = array();
		$sql  = $this->sql_select().',
				MAX(invoices.date) AS last_invoice,
				SUM(payments.value) AS paid,
				invoices.value - SUM(payments.value) AS net_value'
			.$this->sql_from()
			.$this->sql_joins().'
			WHERE invoices.date < \''.date('Y-m-d').'\''
			.$this->sql_group_by()
			.$this->sql_order_by().'
			HAVING paid < invoices.value OR paid IS NULL';
		$result = $this->db->query($sql);
		$result->setFetchMode(PDO::FETCH_CLASS,$this->model);
		return $result->fetchAll();
	}

	public function liberate(&$contract)
	{
		$sql = '
			UPDATE '.$this->table.'
			SET status = \'L\'
			WHERE contract_id = \''.addslashes($contract->contract_id).'\'';
		if ($this->db->query($sql)) {
			$contract->status = 'L';
			return true;
		} else {
			return false;
		}
	}

	public function block(&$contract)
	{
		$sql = '
			UPDATE '.$this->table.'
			SET status = \'B\'
			WHERE contract_id = \''.addslashes($contract->contract_id).'\'';
		if ($this->db->query($sql)) {
			$contract->status = 'B';
			return true;
		} else {
			return false;
		}
	}
}

?>
