<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2008 LinFox Serviços em Informatica LTDA. <alwin@linfox.com.br>

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once('mdl/bank.php');

$banks = array(
	'001' => 'banco_do_brasil',
	'356' => 'banco_real' );

class banco_do_brasil extends bank
{
	public function __construct()
	{
		parent::__construct();
	}
	
	public function get_code($invoice)
	{
		$carteira = number_fit($invoice->account_carteira,2);
		
		$code  = $this->get_base_code($invoice);               // Base code (19)

		if (strlen($invoice->account_convention) == 8) {
			$code .= '000000';                                 // Leading zeroes (6)
			$code .= $invoice->account_convention;             // Convention (8)
			$code .= number_fit($invoice->invoice_id,9);       // Payment number (9)
			$code .= number_fit($carteira,2);                  // Carteira (2)
		} elseif (strlen($invoice->account_convention) == 7) { 
			$code .= '000000';                                 // Leading zeroes (6)
			$code .= $invoice->account_convention;             // Convention (7)
			$code .= number_fit($invoice->invoice_id,10);      // Payment number (10)
			$code .= number_fit($carteira,2);                  // Carteira (2)
		} elseif (strlen($invoice->account_convention) == 6) {
			$code .= $invoice->account_convention;             // Convention (6)
			$code .= number_fit($invoice->invoice_id,5);       // Payment number (5)
			$code .= number_fit($invoice->account_agency,4);   // Agency (4)
			$code .= number_fit($invoice->account_number,8);   // Account Number (8)
			$code .= number_fit($carteira,2);                  // Carteira (2)
		}
		$code = $this->calc_dv($code);  // Insert dv
		return $code;
	}
	
	public function get_payment_number($invoice)
	{
		$convention = $invoice->account_convention; 
		if (strlen($convention) == 8) {
			$payment_number = number_fit($invoice->invoice_id,9);
		} elseif (strlen($convention) == 7) {
			$payment_number = number_fit($invoice->invoice_id,10);
		} elseif (strlen($convention) == 6) {
			$payment_number = number_fit($invoice->invoice_id,5);
		}
		return $convention.$payment_number .' - '. $this->modulo_11($convention.$payment_number);
	}
}

class banco_real extends bank
{
	public function __construct()
	{
		parent::__construct();
	}
	
	public function get_code($invoice)
	{
		$payment_number = $this->get_payment_number($invoice);
		$agency = number_fit($invoice->account_agency, 4);
		$account = number_fit($invoice->account_number, 7);
		$cobranca_dv = $this->modulo_10($payment_number.$agency.$account);
		
		$code  = $this->get_base_code($invoice); // Base code (19)
		$code .= $agency;               // Agency number (4)
		$code .= $account;              // Account number (7)
		$code .= $cobranca_dv;          // Invoice parity digit (1)
		$code .= $payment_number;       // Payment number (13)
		$code  = $this->calc_dv($code); // Insert dv

		return $code;
	}
	
	public function get_payment_number($invoice)
	{
		return substr($invoice->date,2,2) . substr($invoice->date,5,2) . number_fit($invoice->invoice_id,9);
	}
}

?>