<?php

/*	Vulpes - Gateway Access Manager
	Copyright © 2007, 2008 LinFox Serviços em Informatica LTDA.

	This file is part of Vulpes.

	Vulpes is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Vulpes is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Vulpes.  If not, see <http://www.gnu.org/licenses/>. */

require_once 'dao/client_account_dao.php';
require_once 'dao/network_dao.php';
require_once 'dao/contract_dao.php';

class iptables_plugin extends plugin
{
	public $actions = array(
		'client' => array('update','insert','remove'),
		'client_account' => array('update','insert','remove'),
		'config' => array('update','insert','remove'),
		'contract' => array('update','insert','remove'),
		'network' => array('update','insert','remove'),
		'schedule' => array('update','insert','remove'),
		'service' => array('update','insert','remove') );

	public function call()
	{
		$config = config::get_instance();

		$commands = array();
		$iptables = '';
		$iptables_nat = '';
		$iptables_mangle = '';

		$client_account_dao = new client_account_dao();
		$network_dao = new network_dao();
		$accounts = $client_account_dao->fetch_all();
		$networks = $network_dao->fetch_all();

		// tcrc is going to be a shell script.
		$tcrc = "#!/bin/sh\n";

		// Create a root qdisc for every network with a real network interface
		foreach ($networks as $network) {
			if (strpos($network->iface,':')) continue;
			$tcrc .= "tc qdisc del dev $network->iface root\n";
			$tcrc .= "tc qdisc add dev $network->iface root handle 1: htb default 3\n";
			$tcrc .= "tc class add dev $network->iface parent 1: classid 1:1 htb rate ".$network->rate."kbit\n";

			$tcrc .= "tc class add dev $network->iface parent 1:1 classid 1:3 htb rate ".$network->rate_default."kbit\n";
			$tcrc .= "tc qdisc add dev $network->iface parent 1:3 handle 3: sfq perturb 10\n";
		}

		// Going to use this to see witch contracts I've allready created a qdisc for
		$used_contracts = array();
		$i = 16;

		foreach ($accounts as $account) {
			if (!$account->ipv4 || !$account->mac) continue;
			if (!$account->scheduled) continue;
			if ( !($account->contract_status == 'L'
			|| $account->contract_statys == 'P') )
				continue;

			if (!array_key_exists($account->contract_id,$used_contracts)) {
				$used_contracts[$account->contract_id] = array(
					'handle_up'          => strtoupper(dechex($i)),
					'handle_down'        => strtoupper(dechex($i + 32768)),
					'handle_mangle_up'   => $i,
					'handle_mangle_down' => $i + 32768 );

				$account->handle_up = $used_contracts[$account->contract_id]['handle_up'];
				$account->handle_down = $used_contracts[$account->contract_id]['handle_down'];
				$account->handle_mangle_up = $used_contracts[$account->contract_id]['handle_mangle_up'];
				$account->handle_mangle_down = $used_contracts[$account->contract_id]['handle_mangle_down'];

				$tcrc .= "tc class add dev $config->iface parent 1:1 classid 1:$account->handle_up htb rate ".$account->service_upload."kbit\n";
				$tcrc .= "tc qdisc add dev $config->iface parent 1:$account->handle_up handle $account->handle_up: sfq perturb 10\n";
				$tcrc .= "tc class add dev $account->network_iface parent 1:1 classid 1:$account->handle_down htb rate ".$account->service_download."kbit\n";
				$tcrc .= "tc qdisc add dev $account->network_iface parent 1:$account->handle_down handle $account->handle_down: sfq perturb 10\n";
				$i++;
			} else {
				$account->handle_up = $used_contracts[$account->contract_id]['handle_up'];
				$account->handle_down = $used_contracts[$account->contract_id]['handle_down'];
				$account->handle_mangle_up = $used_contracts[$account->contract_id]['handle_mangle_up'];
				$account->handle_mangle_down = $used_contracts[$account->contract_id]['handle_mangle_down'];
			}

			$tcrc .= "tc filter add dev $config->iface parent 1: protocol ip prio 1 handle $account->handle_mangle_up fw flowid 1:$account->handle_up\n";
			$tcrc.= "tc filter add dev $account->network_iface parent 1: protocol ip prio 1 handle $account->handle_mangle_down fw flowid 1:$account->handle_down\n";

			$iptables_nat .= "-A PREROUTING -s $account->ipv4 -p tcp --dport 80";
			if ($account->service_bind_mac)
				$iptables_nat .= ' -m mac --mac-source '.strtoupper($account->mac);
			if ($account->service_proxy_port)
				$iptables_nat .= " -j REDIRECT --to-port $account->service_proxy_port\n";
			else $iptables_nat .= " -j ACCEPT\n";

			$iptables_mangle .= "-A PREROUTING -i $account->network_iface -s $account->ipv4 -j MARK --set-mark $account->handle_mangle_up\n";
			$iptables_mangle .= "-A INPUT -i $account->network_iface -s $account->ipv4 -j MARK --set-mark $account->handle_mangle_up\n";
			$iptables_mangle .= "-A FORWARD -i $config->iface -o $account->network_iface -d $account->ipv4 -j MARK --set-mark $account->handle_mangle_down\n";
			$iptables_mangle .= "-A OUTPUT -o $account->network_iface -d $account->ipv4 -j MARK --set-mark $account->handle_mangle_down\n";
			
			if ($account->service_ttl_limit) {
				$iptables_filter .= "-A FORWARD -s $account->ipv4 -m ttl --ttl-gt 128 -j DROP\n";
				$iptables_filter .= "-A FORWARD -s $account->ipv4 -m ttl --ttl-gt 64 -j DROP\n";
				$iptables_filter .= "-A FORWARD -s $account->ipv4 -m ttl --ttl-lt ".(64 - $account->service_ttl_limit)." -j DROP\n";
			}

			$iptables_filter .= "-A FORWARD -i $account->network_iface -s $account->ipv4";
			if ($account->service_bind_mac) $iptables_filter .= ' -m mac --mac-source '.strtoupper($account->mac);
			$iptables_filter .= " -j ACCEPT\n";

			$iptables_filter .= "-A FORWARD -i $config->iface -d $account->ipv4 -j ACCEPT\n";

			$iptables_filter .= "-A INPUT -i $account->network_iface -s $account->ipv4";
			if ($account->service_bind_mac) $iptables_filter .= ' -m mac --mac-source '.strtoupper($account->mac);
			$iptables_filter .= " -j ACCEPT\n";

			$iptables_filter .= "-A OUTPUT -o $account->network_iface -d $account->ipv4 -j ACCEPT\n";
		}
		$iptables = stripcslashes($config->iptables);
		$iptables = str_replace('{vulpes_nat}',$iptables_nat,$iptables);
		$iptables = str_replace('{vulpes_mangle}',$iptables_mangle,$iptables);
		$iptables = str_replace('{vulpes_filter}',$iptables_filter,$iptables);
		$file_iptables = fopen('/etc/firewall','w');
		fwrite($file_iptables,$iptables);
		fclose($file_iptables);
		exec('/sbin/iptables-restore < /etc/firewall > /dev/null &');

		$file_tc = fopen('/etc/tcrc','w');
		fwrite($file_tc, $tcrc);
		fclose($file_tc);
		exec('sh /etc/tcrc > /dev/null &');

		return true;
	}
}

?>