<?php

include 'config.php';
include 'libfox.php';

$db = db::get_instance();

$source_db = 'vulpes03';
$target_db = 'vulpes';
$admin_id = 1;
$iface = 'eth0';

// Migrate config
$sql = "
INSERT INTO $target_db.config (
	config_id, sitename, domain, dhcpd_header, iptables, ip, cp_port, language,
	bandwith, iface, proxy_port, invoice_type, invoice_payday_offset,
	payment_delay, log_prune, template, block_overdue, cp_header, cp_footer )
SELECT
	config_id, sitename, domain, dhcpd_header, iptables, ip, cp_port, language,
	bandwith, iface, proxy_port, NULL, NULL,
	payment_delay, log_prune, template, block_overdue, cp_header, cp_footer
FROM $source_db.config
ON DUPLICATE KEY UPDATE
	config_id = VALUES(config_id),
	sitename = VALUES(sitename),
	domain = VALUES(domain),
	dhcpd_header = VALUES(dhcpd_header),
	iptables = VALUES(iptables),
	ip = VALUES(ip),
	cp_port = VALUES(cp_port),
	cp_header = VALUES(cp_header),
	cp_footer = VALUES(cp_footer),
	language = VALUES(language),
	bandwith = VALUES(bandwith),
	iface = VALUES(iface),
	proxy_port = VALUES(proxy_port),
	payment_delay = VALUES(payment_delay),
	log_prune = VALUES(log_prune),
	template = VALUES(template),
	block_overdue = VALUES(block_overdue),
	proxy_port = VALUES(proxy_port)";
$db->query($sql);

// Migrate networks
$sql = "
INSERT INTO $target_db.networks (
	iface, name, rate, rate_default )
SELECT
	plans.iface,
	CONCAT('Network ',plans.iface),
	100000,
	100
FROM $source_db.plans
WHERE plans.iface != ''
GROUP BY plans.iface";
$db->query($sql);

$sql = "
INSERT INTO $target_db.networks (
	iface, name, rate, rate_default )
SELECT
	iface, 'WAN', 100000, 100
FROM $source_db.config
GROUP BY iface
ON DUPLICATE KEY UPDATE networks.name = 'WAN'";
$db->query($sql);


// Migrate plans/services
$sql = "
UPDATE $source_db.plans, $target_db.networks
SET plans.iface = '$iface'
WHERE plans.iface != networks.iface";
$db->query($sql);

$sql = "
INSERT INTO $target_db.services (
	service_id, network_id, name, description, price, price_ip, payment,
	download, upload, auth, bind_mac, proxy_port, ttl_limit )
SELECT plans.plan_id, networks.network_id, plans.name, plans.description,
	plans.price, 0, 1, plans.download, plans.upload, plans.auth,
	plans.bind_mac, plans.proxy_port, plans.ttl_limit
FROM $source_db.plans
INNER JOIN $target_db.networks ON (
	networks.iface = plans.iface )";
$db->query($sql);

// Migrate users
$sql = "
INSERT INTO $target_db.users ( 
	user_id,
	username,
	name,
	email,
	language,
	password,
	admin )
SELECT
	user_id,
	username,
	name,
	email,
	'en_us',
	password,
	admin
FROM $source_db.users
ON DUPLICATE KEY UPDATE
	user_id = VALUES(user_id),
	username = VALUES(username),
	name = VALUES(name),
	email = VALUES(email),
	language = 'en_us',
	password = VALUES(password),
	admin = VALUES(admin)";
$db->query($sql);

// Migrate user groups
$sql = "INSERT INTO $target_db.user_groups ( 
	group_id,
	name,
	description )
SELECT
	group_id,
	name,
	description
FROM $source_db.user_groups
ON DUPLICATE KEY UPDATE
	group_id = VALUES(group_id),
	name = VALUES(name),
	description = VALUES(description)";
$db->query($sql);

$sql = "INSERT INTO $target_db.user_members ( 
	member_id,
	user_id,
	group_id )
SELECT
	user_members.member_id,
	user_members.user_id,
	user_members.group_id
FROM $source_db.user_members
INNER JOIN $target_db.users ON (
	users.user_id = user_members.user_id )
INNER JOIN $target_db.user_groups ON (
	user_groups.group_id = user_members.group_id )
ON DUPLICATE KEY UPDATE
	member_id = VALUES(member_id),
	user_id = VALUES(user_id),
	group_id = VALUES(group_id)";
$db->query($sql);

$sql = "INSERT INTO $target_db.user_permissions ( 
	permission_id,
	group_id,
	mode,
	action )
SELECT
	user_permissions.permission_id,
	user_permissions.group_id,
	user_permissions.mode,
	user_permissions.action
FROM $source_db.user_permissions
INNER JOIN $target_db.user_groups ON (
	user_groups.group_id = user_permissions.group_id )
ON DUPLICATE KEY UPDATE
	permission_id = VALUES(permission_id),
	group_id = VALUES(group_id),
	mode = VALUES(mode),
	action = VALUES(action)";
$db->query($sql);

$sql = "INSERT INTO $target_db.accounts (
	account_id, name, bank_name, bank_code, agency, account_number, carteira,
	currency, owner_name, owner_cpf_cnpj, owner_address,
	owner_city,
	owner_state,
	payment_number,
	payment_info1,
	payment_info2,
	payment_info3,
	payment_info4,
	payment_instructions1,
	payment_instructions2,
	payment_instructions3,
	payment_instructions4 )
SELECT
	account_id,
	name,
	bank_name,
	bank_code,
	agency,
	account_number,
	carteira,
	currency,
	owner_name,
	owner_cpf_cnpj,
	owner_address,
	owner_city,
	owner_state,
	payment_number,
	payment_info1,
	payment_info2,
	payment_info3,
	payment_info4,
	payment_instructions1,
	payment_instructions2,
	payment_instructions3,
	payment_instructions4
FROM $source_db.accounts";
$db->query($sql);

$sql = "
INSERT INTO $target_db.payment_methods (
	method_id,
	account_id,
	name,
	bill,
	description )
SELECT
	account_id,
	account_id,
	name,
	1,
	'Metodo criado automaticamente'
FROM $source_db.accounts";
$db->query($sql);

$sql = "
INSERT INTO $target_db.clients (
	client_id,
	username,
	password,
	name,
	email,
	address1,
	address2,
	district,
	postal_code,
	city,
	state,
	phone,
	mobile,
	fiscal_number,
	identity,
	notes,
	status )
SELECT
	client_id,
	username,
	password,
	name,
	email,
	address,
	address2,
	district,
	postal_code,
	city,
	state,
	phone,
	mobile,
	fiscal_number,
	identity,
	notes,
	status
FROM $source_db.clients";
$db->query($sql);


// Migrate contracts
$sql = "
	UPDATE $source_db.contracts, $target_db.users
	SET contracts.user_id = $admin_id
	WHERE contracts.user_id != users.user_id";
$db->query($sql);
$sql = "
INSERT INTO $target_db.contracts (
	contract_id,
	service_id,
	client_id,
	user_id,
	method_id,
	start,
	end,
	payday,
	status )
SELECT
	contracts.handle,
	contracts.plan_id,
	contracts.client_id,
	contracts.user_id,
	plans.account_id,
	contracts.date,
	contracts.canceled,
	contracts.payday,
	contracts.status
FROM $source_db.contracts
INNER JOIN $target_db.services ON (
	services.service_id = contracts.plan_id )
INNER JOIN $target_db.clients ON (
	clients.client_id = contracts.client_id )
INNER JOIN $target_db.users ON (
	users.user_id = contracts.user_id )
INNER JOIN $source_db.plans ON (
	plans.plan_id = contracts.plan_id )
INNER JOIN $target_db.payment_methods ON (
	payment_methods.method_id = plans.account_id )";
$db->query($sql);

// Migrate invoices
$sql = "
INSERT INTO $target_db.invoices (
	invoice_id,
	client_id,
	contract_id,
	service_id,
	method_id,
	value,
	discount,
	date,
	period_start,
	period_end,
	description,
	comments,
	status )
SELECT
	invoices.invoice_id,
	contracts.client_id,
	invoices.handle,
	contracts.service_id,
	contracts.method_id,
	invoices.value,
	invoices.discount,
	invoices.date,
	invoices.period_start,
	invoices.period_end,
	invoices.description,
	invoices.comments,
	invoices.status
FROM $source_db.invoices
INNER JOIN $target_db.contracts ON (
	contracts.contract_id = invoices.handle )";
$db->query($sql);

// Migrate payments
$sql = "
	UPDATE $source_db.payments, $target_db.users
	SET payments.user_id = $admin_id
	WHERE payments.user_id != users.user_id";
$db->query($sql);
$sql = "
INSERT INTO $target_db.payments (
	payment_id,
	method_id,
	client_id,
	user_id,
	contract_id,
	invoice_id,
	value,
	interest,
	fines,
	date,
	status )
SELECT
	payments.payment_id,
	contracts.method_id,
	contracts.client_id,
	payments.user_id,
	contracts_left.contract_id,
	payments.invoice_id,
	payments.value,
	payments.interest,
	payments.fines,
	payments.date,
	payments.status
FROM $source_db.payments
INNER JOIN $target_db.invoices ON (
	invoices.invoice_id = payments.invoice_id )
INNER JOIN $target_db.contracts ON (
	contracts.contract_id = invoices.contract_id )
LEFT JOIN $target_db.contracts contracts_left ON (
	contracts_left.contract_id = invoices.contract_id )
INNER JOIN $target_db.users ON (
	users.user_id = payments.user_id )";
$db->query($sql);

$sql = "
INSERT INTO $target_db.schedules (
	schedule_id,
	service_id,
	day,
	start,
	end,
	upload,
	download )
SELECT
	schedules.schedule_id,
	services.service_id,
	schedules.day,
	schedules.start,
	schedules.end,
	schedules.upload,
	schedules.download
FROM $source_db.schedules
INNER JOIN $target_db.services ON (
	services.service_id = schedules.plan_id )";
$db->query($sql);

/* $sql = "
INSERT INTO $target_db.client_accounts (
	client_id,
	contract_id,
	network_id,
	username,
	password,
	mac,
	ipv4 )"; */

$sql = "
SELECT
	contracts.client_id AS client_id,
	contracts.handle AS contract_id,
	networks.network_id AS network_id,
	CONCAT(clients.username,contracts.handle) AS username,
	CONCAT(clients.username,'123') AS password,
	contracts.mac AS mac,
	contracts.ip AS ipv4
FROM $source_db.contracts
INNER JOIN $target_db.contracts target_contracts ON (
	target_contracts.contract_id = contracts.handle )
INNER JOIN $target_db.clients ON (
	clients.client_id = contracts.client_id )
INNER JOIN $target_db.services ON (
	services.service_id = contracts.plan_id )
INNER JOIN $target_db.networks ON (
	networks.network_id = services.network_id )";
$result = $db->query($sql);
$result->setFetchMode(PDO::FETCH_OBJ);
$contracts = $result->fetchAll();
foreach ($contracts as &$contract) {
	$mac_hex = mac2hex($contract->mac);
	$ipv4_hex = ip2hex($contract->ipv4);
	$sql = "
		INSERT INTO $target_db.client_accounts (
			client_id, contract_id, network_id, username, password, mac, mac_hex, ipv4, ipv4_hex)
		VALUES (
			$contract->client_id,
			$contract->contract_id,
			$contract->network_id,
			'$contract->username',
			'$contract->password',
			'$contract->mac',
			'$mac_hex',
			'$contract->ipv4',
			'$ipv4_hex' )";
	$db->query($sql);
}

$sql = "
INSERT INTO $target_db.log (
	log_id,
	user_id,
	client_id,
	mode,
	action,
	date_time )
SELECT
	log.log_id,
	log.user_id,
	clients.client_id,
	log.mode,
	log.action,
	log.date_time
FROM $source_db.log
LEFT JOIN $target_db.clients ON (
	clients.client_id = log.client_id )
INNER JOIN $target_db.users ON (
	users.user_id = log.user_id )";
$db->query($sql);

?>
